# include <ohana.h>
# include <gfitsio.h>

int gfits_print (Header *header, char *field, char *mode, int N,...) {
  
  /* this function expects one more argument, the value to be written */

  static char blank[] = " ";
  char string[82], line[80];
  char *p, a;
  va_list argp;  

  va_start (argp, N);
  
  if (mode[0] != '%') {
    fprintf (stderr, "gfits_print: weird mode:  %s\n", mode);
    return (FALSE);
  }

  /* this is supposed to create a new field, not modify an old one.  */
  p = gfits_header_field (header, field, N);
  if (p != NULL) return (FALSE);

  /* find the END of the header */
  p = gfits_header_field (header, "END", 1);
  if (p == NULL) return (FALSE); 

  /* is there enough space for 1 more line? */
  if (header[0].size - (p - (header[0].buffer)) < 2*FT_LINE_LENGTH) {
    header[0].size += FT_RECORD_SIZE;
    REALLOCATE (header[0].buffer, char, header[0].size);
    /* re-find the "END" marker, in case new memory block is used */
    p = gfits_header_field (header, "END", 1);
    if (p == NULL) return (FALSE); 
    memset (p + FT_LINE_LENGTH, ' ', FT_RECORD_SIZE);
  }

  /* push END line back 1 */
  memmove ((p + FT_LINE_LENGTH), p, FT_LINE_LENGTH);
  memset (p, ' ', FT_LINE_LENGTH);

  /* write the new FITS card, setting the comment to be blank */
  /* in these lines, the value field will expand as needed, forcing out the comment
     the total line is limited by snprintf to only 80 chars + EOL */

  /* write the numeric modes */
  if (!strcmp (mode, "%d"))  { snprintf (string, 81, "%-8s= %20d / %46s ",    field, va_arg (argp, int),      blank); }
  if (!strcmp (mode, "%u"))  { snprintf (string, 81, "%-8s= %20d / %46s ",    field, va_arg (argp, unsigned), blank); }
  if (!strcmp (mode, "%ld")) { snprintf (string, 81, "%-8s= %20ld / %46s ",   field, va_arg (argp, long),     blank); }
  if (!strcmp (mode, "%hd")) { snprintf (string, 81, "%-8s= %20d / %46s ",    field, va_arg (argp, int),      blank); }
  if (!strcmp (mode, "%f"))  { snprintf (string, 81, "%-8s= %20.10f / %46s ", field, va_arg (argp, double),   blank); }
  if (!strcmp (mode, "%lf")) { snprintf (string, 81, "%-8s= %20.10f / %46s ", field, va_arg (argp, double),   blank); }
  if (!strcmp (mode, "%e"))  { snprintf (string, 81, "%-8s= %20.10E / %46s ", field, va_arg (argp, double),   blank); }
  if (!strcmp (mode, "%le")) { snprintf (string, 81, "%-8s= %20.10E / %46s ", field, va_arg (argp, double),   blank); }
  if (!strcmp (mode, "%g"))  { snprintf (string, 81, "%-8s= %20.10G / %46s ", field, va_arg (argp, double),   blank); }
  if (!strcmp (mode, "%lg")) { snprintf (string, 81, "%-8s= %20.10G / %46s ", field, va_arg (argp, double),   blank); }

  /* write the boolean mode */
  if (!strcmp (mode, "%t")) {
    a = va_arg (argp, int);
    if (a == 1)
      snprintf (string, 81, "%-8s= %18s T / %46s ", field, blank, blank);
    else
      snprintf (string, 81, "%-8s= %18s F / %46s ", field, blank, blank);
  }

  /* string value.  Quotes must be at least 18 chars apart.  Longer lines will this should be fixed to allow arbitrary string lengths, up to 69 chars */
  if (!strcmp (mode, "%s")) {
    strcpy (line, va_arg (argp, char *));
    line[68] = 0;
    snprintf (string, 81, "%-8s= '%-18s' / %46s ", field, line, blank);
  }

  /* comment type of value.  */
  if (!strcmp (mode, "%S")) {
    /* we are forcing the entry to be <= 69 char long */
    strcpy (line, va_arg (argp, char *));
    line[71] = 0;
    snprintf (string, 81, "%-8s %-71s", field, line);
  }  

  /*
maximal string line:
12345678901234567890123456789012345678901234567890123456789012345678901234567890 .
KEYWORD = '01234567890123456789012345678901234567890123456789012345678901234567'

maximal comment line:
12345678901234567890123456789012345678901234567890123456789012345678901234567890 .
KEYWORD  01234567890123456789012345678901234567890123456789012345678901234567890
  */

  /* can't write the comment in gfits_print - use gfits_modify */
  if (!strcmp (mode, "%C")) return (FALSE);

  strncpy (p, string, 80);
  
  va_end (argp);
  return (TRUE);

}
