## magic.pro : support for the streak removal : -*- sh -*-

# test for required global variables
check.globals

$LOGSUBDIR = $LOGDIR/magic
exec mkdir -p $LOGSUBDIR

### Initialise the books containing the tasks to do
book init magicToTree
book init magicToProcess
book init magicToDS

### Database lists
$magicToTree_DB = 0
$magicToProcess_DB = 0
$magicToDS_DB = 0

### Check status of tasks
macro magic.status
  book listbook magicToTree
  book listbook magicToProcess
  book listbook magicToDS
end

### Reset tasks
macro magic.reset
  book init magicToTree
  book init magicToProcess
  book init magicToDS
end

### Turn tasks on
macro magic.on
  task magic.tree.load
    active true
  end
  task magic.tree.run
    active true
  end
  task magic.process.load
    active true
  end
  task magic.process.run
    active true
  end
  task magic.destreak.load
    active true
  end
  task magic.destreak.run
    active true
  end
end

### Turn tasks off
macro magic.off
  task magic.tree.load
    active false
  end
  task magic.tree.run
    active false
  end
  task magic.process.load
    active false
  end
  task magic.process.run
    active false
  end
  task magic.destreak.load
    active false
  end
  task magic.destreak.run
    active false
  end
end

task	       magic.tree.load
  host         local

  periods      -poll $LOADPOLL
  periods      -exec $LOADEXEC
  periods      -timeout 30
  npending     1

  stdout NULL
  stderr $LOGDIR/magic.tree.log

  task.exec
    if ($DB:n == 0)
      option DEFAULT
      command magictool -totree -limit 20
    else
      # save the DB name for the exit tasks
      option $DB:$magicToTree_DB
      command magictool -totree -limit 20 -dbname $DB:$magicToTree_DB
      $magicToTree_DB ++
      if ($magicToTree_DB >= $DB:n) set magicToTree_DB = 0
    end
  end

  # success
  task.exit    0
    # convert 'stdout' to book format
    ipptool2book stdout magicToTree -key magic_id -uniq -setword dbname $options:0 -setword pantaskState INIT
    if ($VERBOSE > 2)
      book listbook magicToTree
    end

    # delete existing entries in the appropriate pantaskStates
    process_cleanup magicToTree
  end

  # locked list
  task.exit    default
    showcommand failure
  end

  # operation times out?
  task.exit    timeout
    showcommand timeout
  end
end

task	       magic.tree.run
  periods      -poll $RUNPOLL
  periods      -exec $RUNEXEC
  periods      -timeout 60

  task.exec
    book npages magicToTree -var N
    if ($N == 0) break
    if ($NETWORK == 0) break
    
    # look for new images (pantaskState == INIT)
    book getpage magicToTree 0 -var pageName -key pantaskState INIT
    if ("$pageName" == "NULL") break

    book setword magicToTree $pageName pantaskState RUN
    book getword magicToTree $pageName magic_id -var MAGIC_ID
    book getword magicToTree $pageName exp_id -var EXP_ID
    book getword magicToTree $pageName camera -var CAMERA
    book getword magicToTree $pageName workdir -var WORKDIR_TEMPLATE
    book getword magicToTree $pageName dbname -var DBNAME
    book getword magicToTree $pageName tess_id -var TESS_DIR
    book getword magicToTree $pageName ra -var RA
    book getword magicToTree $pageName decl -var DEC

#    set.host.for.camera $CAMERA $MAGIC_ID
#    set.workdir.by.camera $CAMERA $MAGIC_ID $WORKDIR_TEMPLATE $default_host WORKDIR
    host anyhost
    $WORKDIR = $WORKDIR_TEMPLATE

    basename $TESS_DIR -var TESS_ID

    $TESS_ID = "FIXNS"

    sprintf outroot "%s/%s/%s.mgc.%s" $WORKDIR $EXP_ID $EXP_ID $MAGIC_ID

    ## generate output log based on filerule (convert the URI to a PATH)
    $logfile = `ipp_filename.pl --filerule LOG.EXP --camera $CAMERA --class_id $MAGIC_ID --basename $outroot`
    if ("$logfile" == "") 
      echo "WARNING: logfile not defined in magic.tree.run"
      break
    end

    stdout $logfile
    stderr $logfile
    dirname $logfile -var outpath
    mkdir $outpath

    $run = magic_tree.pl --magic_id $MAGIC_ID --camera $CAMERA --tess_id $TESS_DIR --ra $RA --dec $DEC --outroot $outroot --logfile $logfile
    add_standard_args run

    # save the pageName for future reference below
    options $pageName

    # create the command line
    if ($VERBOSE > 1)
      echo command $run
    end
    command $run
  end

  # default exit status
  task.exit    default
    process_exit magicToTree $options:0 $JOB_STATUS
  end

  # operation timed out?
  task.exit    timeout
    showcommand timeout
    book setword magicToTree $options:0 pantaskState TIMEOUT
  end
end

task	       magic.process.load
  host         local

  periods      -poll $LOADPOLL
  periods      -exec $LOADEXEC
  periods      -timeout 30
  npending     1

  stdout NULL
  stderr $LOGDIR/magic.process.log

  task.exec
    if ($DB:n == 0)
      option DEFAULT
      command magictool -toprocess -limit 20
    else
      # save the DB name for the exit tasks
      option $DB:$magicToProcess_DB
      command magictool -toprocess -limit 20 -dbname $DB:$magicToProcess_DB
      $magicToProcess_DB ++
      if ($magicToProcess_DB >= $DB:n) set magicToProcess_DB = 0
    end
  end

  # success
  task.exit    0
    # convert 'stdout' to book format
    ipptool2book stdout magicToProcess -key magic_id:node -uniq -setword dbname $options:0 -setword pantaskState INIT
    if ($VERBOSE > 2)
      book listbook magicToProcess
    end

    # delete existing entries in the appropriate pantaskStates
    process_cleanup magicToProcess
  end

  # locked list
  task.exit    default
    showcommand failure
  end

  # operation times out?
  task.exit    timeout
    showcommand timeout
  end
end

task	       magic.process.run
  periods      -poll $RUNPOLL
  periods      -exec $RUNEXEC
  periods      -timeout 60

  task.exec
    book npages magicToProcess -var N
    if ($N == 0) break
    if ($NETWORK == 0) break
    
    # look for new images (pantaskState == INIT)
    book getpage magicToProcess 0 -var pageName -key pantaskState INIT
    if ("$pageName" == "NULL") break

    book setword magicToProcess $pageName pantaskState RUN
    book getword magicToProcess $pageName magic_id -var MAGIC_ID
    book getword magicToProcess $pageName exp_id -var EXP_ID
    book getword magicToProcess $pageName node -var NODE
    book getword magicToProcess $pageName camera -var CAMERA
    book getword magicToProcess $pageName workdir -var WORKDIR_TEMPLATE
    book getword magicToProcess $pageName dbname -var DBNAME

#    set.host.for.camera $CAMERA $MAGIC_ID
#    set.workdir.by.camera $CAMERA $MAGIC_ID $WORKDIR_TEMPLATE $default_host WORKDIR
    host anyhost
    $WORKDIR = $WORKDIR_TEMPLATE

    sprintf outroot "%s/%s/%s.mgc.%s.%s" $WORKDIR $EXP_ID $EXP_ID $MAGIC_ID $NODE

    ## generate output log based on filerule (convert the URI to a PATH)
    $logfile = `ipp_filename.pl --filerule LOG.EXP --camera $CAMERA --class_id $MAGIC_ID --basename $outroot`
    if ("$logfile" == "") 
      echo "WARNING: logfile not defined in magic.process.run"
      break
    end

    stdout $logfile
    stderr $logfile
    dirname $logfile -var outpath
    mkdir $outpath

    $run = magic_process.pl --magic_id $MAGIC_ID --camera $CAMERA --node $NODE --outroot $outroot --logfile $logfile
    add_standard_args run

    # save the pageName for future reference below
    options $pageName

    # create the command line
    if ($VERBOSE > 1)
      echo command $run
    end
    command $run
  end

  # default exit status
  task.exit    default
    process_exit magicToProcess $options:0 $JOB_STATUS
  end

  # operation timed out?
  task.exit    timeout
    showcommand timeout
    book setword magicToProcess $options:0 pantaskState TIMEOUT
  end
end

task	       magic.destreak.load
  host         local

  periods      -poll $LOADPOLL
  periods      -exec 60
  periods      -timeout 30
  npending     1

  stdout NULL
  stderr $LOGDIR/magic.destreak.log

  task.exec
    if ($DB:n == 0)
      option DEFAULT
      command magicdstool -todestreak -limit 20
    else
      # save the DB name for the exit tasks
      option $DB:$magicToDS_DB
      command magicdstool -todestreak -limit 20 -dbname $DB:$magicToDS_DB
      $magicToDS_DB ++
      if ($magicToDS_DB >= $DB:n) set magicToDS_DB = 0
    end
  end

  # success
  task.exit    0
    # convert 'stdout' to book format
    ipptool2book stdout magicToDS -key magic_ds_id:component -uniq -setword dbname $options:0 -setword pantaskState INIT
    if ($VERBOSE > 2)
      book listbook magicToDS
    end

    # delete existing entries in the appropriate pantaskStates
    process_cleanup magicToDS
  end

  # locked list
  task.exit    default
    showcommand failure
  end

  # operation times out?
  task.exit    timeout
    showcommand timeout
  end
end

task	       magic.destreak.run
  periods      -poll $RUNPOLL
  periods      -exec $RUNEXEC
  periods      -timeout 60

  task.exec
    book npages magicToDS -var N
    if ($N == 0) break
    if ($NETWORK == 0) break
    
    # look for new images (pantaskState == INIT)
    book getpage magicToDS 0 -var pageName -key pantaskState INIT
    if ("$pageName" == "NULL") break

    book setword magicToDS $pageName pantaskState RUN
    book getword magicToDS $pageName exp_id -var EXP_ID
    book getword magicToDS $pageName magic_ds_id -var MAGIC_DS_ID
    book getword magicToDS $pageName camera -var CAMERA
    book getword magicToDS $pageName streaks_uri -var STREAKS
    book getword magicToDS $pageName stage -var STAGE
    book getword magicToDS $pageName stage_id -var STAGE_ID
    book getword magicToDS $pageName component -var COMPONENT
    book getword magicToDS $pageName uri -var URI
    book getword magicToDS $pageName path_base -var PATH_BASE
    book getword magicToDS $pageName cam_path_base -var CAM_PATH_BASE
    book getword magicToDS $pageName outroot -var OUTROOT
    book getword magicToDS $pageName recoveryroot -var RECROOT
    book getword magicToDS $pageName re_place -var REPLACE
    book getword magicToDS $pageName remove -var REMOVE
    book getword magicToDS $pageName dbname -var DBNAME

    sprintf logfile "%s/%s.mds.%s.%s.%s.log" $OUTROOT $EXP_ID $MAGIC_DS_ID $STAGE_ID $COMPONENT

    host anyhost

    # TODO: do not add recoveryroot, replace, or remove if they are null or zero

    $run = magic_destreak.pl --magic_ds_id $MAGIC_DS_ID --camera $CAMERA --streaks $STREAKS --stage $STAGE --stage_id $STAGE_ID --component $COMPONENT --uri $URI --path_base $PATH_BASE --cam_path_base $CAM_PATH_BASE --outroot $OUTROOT --logfile $logfile
    # --recoveryroot $RECROOT --replace $REPLACE --remove $REMOVE 
    add_standard_args run

    # save the pageName for future reference below
    options $pageName

    # create the command line
    if ($VERBOSE > 1)
      echo command $run
    end
    command $run
  end

  # default exit status
  task.exit    0
    process_exit magicToDS $options:0 $JOB_STATUS
   end

  # locked list
  task.exit    default
    showcommand failure
    process_exit magicToDS $options:0 $JOB_STATUS
  end

  # operation timed out?
  task.exit    timeout
    showcommand timeout
    book setword magicToDS $options:0 pantaskState TIMEOUT
  end
end

