/** @file ppSubDefineOutput.c
 *
 *  @brief
 *
 *  @ingroup ppSub
 *
 *  @author IfA
 *  @version $Revision: 1.2 $ $Name: ipp-magic-v0-0 $
 *  @date $Date: 2009/02/06 01:37:17 $
 *  Copyright 2009 Institute for Astronomy, University of Hawaii
 */

#include "ppSub.h"

bool ppSubDefineOutput (pmConfig *config, const pmFPAview *view) {

    bool mdok;

    // generate an output readout 
    pmCell *outCell = pmFPAfileThisCell(config->files, view, "PPSUB.OUTPUT"); // Output cell
    pmReadout *outRO = pmReadoutAlloc(outCell); // Output readout: subtraction
    pmFPA *outFPA = outCell->parent->parent; // Output FPA
    pmHDU *outHDU = outFPA->hdu; // Output HDU
    if (!outHDU->header) {
        outHDU->header = psMetadataAlloc();
    }

    // convolved input images
    pmReadout *inConv = pmFPAfileThisReadout(config->files, view, "PPSUB.INPUT.CONV"); // Input readout
    pmReadout *refConv = pmFPAfileThisReadout(config->files, view, "PPSUB.REF.CONV"); // Reference readout

    // Add kernel descrption to header.  We don't know which readout gets the kernels because
    // it depends on which is larger (the choice is set in ppSubMatchPSFs)
    bool inputHasKernel = true;
    pmSubtractionKernels *kernels = psMetadataLookupPtr(&mdok, inConv->analysis, PM_SUBTRACTION_ANALYSIS_KERNEL); // Subtraction kernel
    if (!kernels) {
        kernels = psMetadataLookupPtr(&mdok, refConv->analysis, PM_SUBTRACTION_ANALYSIS_KERNEL);
	inputHasKernel = false;
    }
    if (!kernels) {
        psError(PS_ERR_UNEXPECTED_NULL, true, "Unable to find SUBTRACTION.KERNEL");
        psFree(inConv);
        psFree(refConv);
        psFree(outRO);
        return false;
    }
    psMetadataAddStr(outHDU->header, PS_LIST_TAIL, "PPSUB.KERNEL", 0, "Subtraction kernel", kernels->description);

    if (inputHasKernel) {
	outRO->analysis = psMetadataCopy(outRO->analysis, inConv->analysis);
    } else {
	outRO->analysis = psMetadataCopy(outRO->analysis, refConv->analysis);
    } 

#ifdef TESTING
    psImage *kernelImage = NULL;

    if (inputHasKernel) {
	kernelImage = psMetadataLookupPtr(&mdok, inConv->analysis, "SUBTRACTION.KERNEL.IMAGE"); // Image of the kernels
    } else {
        kernelImage = psMetadataLookupPtr(&mdok, refConv->analysis, "SUBTRACTION.KERNEL.IMAGE");
    }

    psFits *fits = psFitsOpen("kernel.fits", "w");
    psFitsWriteImage(fits, NULL, kernelImage, 0, NULL);
    psFitsClose(fits);
#endif

    return true;
}
