/** @file ppSubMakePSF.c
 *
 *  @brief
 *
 *  @ingroup ppSub
 *
 *  @author IfA
 *  @version $Revision: 1.2 $ $Name: ipp-magic-v0-0 $
 *  @date $Date: 2009/02/06 01:37:17 $
 *  Copyright 2009 Institute for Astronomy, University of Hawaii
 */

#include "ppSub.h"

/**
 * Photometry stage 1: measure the PSF from the minuend image 
 */
bool ppSubMakePSF (pmConfig *config, const pmFPAview *view) {

    bool mdok = false;

    // Photometry is to be performed in two stages:
    // 1. Measure the PSF using the PSF-matched images
    // 2. Find and measure sources on the subtracted image
    psTimerStart("PPSUB_PHOT");

    // Look up recipe values
    psMetadata *recipe = psMetadataLookupMetadata(NULL, config->recipes, PPSUB_RECIPE); // Recipe for ppSub
    psAssert(recipe, "We checked this earlier, so it should be here.");

    psMetadata *psphotRecipe = psMetadataLookupMetadata(NULL, config->recipes, PSPHOT_RECIPE); // Recipe for psphot
    psAssert(recipe, "We checked this earlier, so it should be here.");

    if (!psMetadataLookupBool(NULL, recipe, "PHOTOMETRY")) return true;

    bool reverse = psMetadataLookupBool(NULL, config->arguments, "REVERSE"); // Reverse sense of subtraction?

    pmReadout *minuend = NULL;
    pmFPAfile *minuendFile = NULL;
    if (reverse) {
	minuend = pmFPAfileThisReadout(config->files, view, "PPSUB.REF.CONV");
	minuendFile = psMetadataLookupPtr(&mdok, config->files, "PPSUB.REF.CONV");
    } else {
	minuend = pmFPAfileThisReadout(config->files, view, "PPSUB.INPUT.CONV");
	minuendFile = psMetadataLookupPtr(&mdok, config->files, "PPSUB.INPUT.CONV");
    }

    // supply the minuend pmFPAfile to psphot as PSPHOT.INPUT:
    psMetadataAddPtr (config->files, PS_LIST_TAIL, "PSPHOT.INPUT", PS_DATA_UNKNOWN | PS_META_REPLACE, "psphot input : view on another pmFPAfile", minuendFile);

    // old-style variance renormalization
    if (!ppSubReadoutRenormPhot (config, recipe, minuend)) {
	psError(PS_ERR_UNKNOWN, false, "failure in renormalization");
	return false;
    }

    // extract the loaded sources from the associated readout
    pmReadout *sourcesRO = pmFPAfileThisReadout(config->files, view, "PPSUB.SOURCES");
    psArray *sources = psMetadataLookupPtr(&mdok, sourcesRO->analysis, "PSPHOT.SOURCES");

    // generate PSF from the supplied sources (assumes image is background-subtracted)
    if (!psphotReadoutFindPSF(config, view, sources)) {
	psError(PS_ERR_UNKNOWN, false, "Unable to perform photometry on subtracted image.");
	return false;
    }

    // Record the FWHM in the output header
    pmReadout *outRO = pmFPAfileThisReadout(config->files, view, "PPSUB.OUTPUT"); // Output readout
    pmHDU *hdu = pmHDUFromCell(outRO->parent); // HDU with header
    psMetadataItemSupplement(hdu->header, psphotRecipe, "FWHM_MAJ");
    psMetadataItemSupplement(hdu->header, psphotRecipe, "FWHM_MIN");

    return true;
}

// XXX we used to need this, is it still needed?

// pmCell *photCell = pmFPAfileThisCell(config->files, view, "PSPHOT.INPUT");
// pmCellFreeReadouts(photCell);

// if (!pmFPAfileDropInternal(config->files, "PSPHOT.BACKMDL") ||
//     !pmFPAfileDropInternal (config->files, "PSPHOT.BACKMDL.STDEV") ||
//     !pmFPAfileDropInternal (config->files, "PSPHOT.BACKGND")) {
//     psError(PS_ERR_UNKNOWN, false, "Unable to drop PSPHOT internal files.");
//     return false;
// }

// Blow away the sources psphot found --- they're irrelevant for the subtraction
// XXX is this still needed?  These are now probably not being set 
// pmReadout *photRO = pmFPAviewThisReadout(view, photFile->fpa); // Readout with sources
// psMetadataRemoveKey(photRO->analysis, "PSPHOT.SOURCES");
// psMetadataRemoveKey(photRO->analysis, "PSPHOT.HEADER");

