/* @file  pmObjects.h
 *
 * The process of finding, measuring, and classifying astronomical sources on
 * images is one of the critical tasks of the IPP or any astronomical software
 * system. This file will define structures and functions related to the task
 * of source detection and measurement. The elements defined in this section
 * are generally low-level components which can be connected together to
 * construct a complete object measurement suite.
 *
 * @author GLG, MHPCC
 *
 * @version $Revision: 1.8 $ $Name: ipp-magic-v0-0 $
 * @date $Date: 2007/01/24 02:54:15 $
 * Copyright 2004 Maui High Performance Computing Center, University of Hawaii
 */

#ifndef PM_OBJECTS_H
#define PM_OBJECTS_H

/// @addtogroup Objects Object Detection / Analysis Functions
/// @{

#include <stdio.h>
#include <math.h>
#include <pslib.h>

/**
 *
 * This function converts the source classification into the closest available
 * approximation to the Dophot classification scheme:
 * XXX EAM : fix this to use current source classification scheme
 *
 * PM_SOURCE_DEFECT: 8
 * PM_SOURCE_SATURATED: 8
 * PM_SOURCE_SATSTAR: 10
 * PM_SOURCE_PSFSTAR: 1
 * PM_SOURCE_GOODSTAR: 1
 * PM_SOURCE_POOR_FIT_PSF: 7
 * PM_SOURCE_FAIL_FIT_PSF: 4
 * PM_SOURCE_FAINTSTAR: 4
 * PM_SOURCE_GALAXY: 2
 * PM_SOURCE_FAINT_GALAXY: 2
 * PM_SOURCE_DROP_GALAXY: 2
 * PM_SOURCE_FAIL_FIT_GAL: 2
 * PM_SOURCE_POOR_FIT_GAL: 2
 * PM_SOURCE_OTHER: ?
 *
 */
int pmSourceDophotType(
    pmSource *source                    ///< Add comment.
);


/** pmSourceSextractType()
 *
 * This function converts the source classification into the closest available
 * approximation to the Sextractor classification scheme. the correspondence is
 * not yet defined (TBD) .
 *
 * XXX: Must code this.
 *
 */
int pmSourceSextractType(
    pmSource *source                    ///< Add comment.
);

/// @}
#endif
